@@ -1,12 +1,13 @@
 "Implements SingleEquationConstraint"
 from operator import le, ge, eq
 from ..small_scripts import try_str_without
-from ..repr_conventions import ReprMixin
+from ..repr_conventions import ReprMixin, UNICODE_EXPONENTS
 
 
 class SingleEquationConstraint(ReprMixin):
     "Constraint expressible in a single equation."
     latex_opers = {"<=": "\\leq", ">=": "\\geq", "=": "="}
+    unicode_opers = {"<=": "≤", ">=": "≥", "=": "="}
     func_opers = {"<=": le, ">=": ge, "=": eq}
 
     def __init__(self, left, oper, right):
@@ -20,7 +21,11 @@ class SingleEquationConstraint(ReprMixin):
         if len(rlines) > 1:
             indent = len("%s %s " % (leftstr.split("\n")[-1], self.oper))
             rightstr = ("\n" + " "*indent).join(rlines)
-        return "%s %s %s" % (leftstr, self.oper, rightstr)
+        if UNICODE_EXPONENTS:
+            oper = self.unicode_opers[self.oper]
+        else:
+            oper = self.oper
+        return "%s %s %s" % (leftstr, oper, rightstr)
 
     def latex(self, excluded=("units")):
         "Latex representation without attributes in excluded list"
