@@ -3,12 +3,29 @@ from collections.abc import Iterable
 import numpy as np
 
 
-def appendsolwarning(msg, data, result, category="uncategorized"):
-    "Append a particular category of warnings to a solution."
+def broadcast_substitution(key, array):
+    "Broadcasts input into the shape of a given key"
+    return np.broadcast_to(array, reversed(key.key.shape)).T
+
+
+def veclinkedfn(linkedfn, i):
+    "Generate an indexed linking function."
+    def newlinkedfn(c):
+        "Linked function that pulls out a particular index"
+        return np.array(linkedfn(c))[i]
+    return newlinkedfn
+
+
+def initsolwarning(result, category="uncategorized"):
+    "Creates a results dictionary for a particular category of warning."
     if "warnings" not in result:
         result["warnings"] = {}
     if category not in result["warnings"]:
         result["warnings"][category] = []
+
+
+def appendsolwarning(msg, data, result, category="uncategorized"):
+    "Append a particular category of warnings to a solution."
     result["warnings"][category].append((msg, data))
 
 
@@ -23,8 +40,8 @@ def isnan(element):
 
 def maybe_flatten(value):
     "Extract values from 0-d numpy arrays, if necessary"
-    if hasattr(value, "shape") and not value.shape:
-        return value.flatten()[0]  # 0-d numpy arrays
+    if hasattr(value, "size") and value.size == 1:
+        return value.item()
     return value
 
 
