@@ -130,12 +130,10 @@ class TestConstraint(unittest.TestCase):
         self.assertEqual(c.as_hmapslt1({}), [(y**2/x).hmap])
         self.assertEqual(c.left, x)
         self.assertEqual(c.right, y**2)
-        self.assertTrue(">=" in str(c))
         c = PosynomialInequality(x, "<=", y**2)
         self.assertEqual(c.as_hmapslt1({}), [(x/y**2).hmap])
         self.assertEqual(c.left, x)
         self.assertEqual(c.right, y**2)
-        self.assertTrue("<=" in str(c))
         self.assertEqual(type((1 >= x).latex()), str)
 
     def test_oper_overload(self):
@@ -146,7 +144,6 @@ class TestConstraint(unittest.TestCase):
         self.assertEqual(c.as_hmapslt1({}), [(1/y + x**2/y).hmap])
         self.assertEqual(c.left, y)
         self.assertEqual(c.right, 1 + x**2)
-        self.assertTrue(">=" in str(c))
         # same constraint, switched operator direction
         c2 = (1 + x**2 <= y)  # same as c
         self.assertEqual(c2.as_hmapslt1({}), c.as_hmapslt1({}))
@@ -318,9 +315,9 @@ class TestLoose(unittest.TestCase):
         m = Model(x, [Loose([x >= x_min]),
                       x >= 1])
         sol = m.solve(verbosity=0)
-        self.assertIs(
-            sol["warnings"]["Unexpectedly Tight Constraints"][0][1], m[0][0])
-        self.assertAlmostEqual(m[0][0].relax_sensitivity, +1, 3)
+        warndata = sol["warnings"]["Unexpectedly Tight Constraints"][0][1]
+        self.assertIs(warndata[-1], m[0][0])
+        self.assertAlmostEqual(warndata[0], +1, 3)
         m.substitutions[x_min] = 0.5
         self.assertAlmostEqual(m.solve(verbosity=0)["cost"], 1)
 
@@ -334,9 +331,9 @@ class TestLoose(unittest.TestCase):
         m = Model(x*y, [Loose([x >= y]),
                         x >= x_min, y >= y_min, sig_constraint])
         sol = m.localsolve(verbosity=0)
-        self.assertIs(
-            sol["warnings"]["Unexpectedly Tight Constraints"][0][1], m[0][0])
-        self.assertAlmostEqual(m[0][0].relax_sensitivity, +1, 3)
+        warndata = sol["warnings"]["Unexpectedly Tight Constraints"][0][1]
+        self.assertIs(warndata[-1], m[0][0])
+        self.assertAlmostEqual(warndata[0], +1, 3)
         m.substitutions[x_min] = 2
         m.substitutions[y_min] = 1
         self.assertAlmostEqual(m.localsolve(verbosity=0)["cost"], 2.5, 5)
@@ -352,9 +349,9 @@ class TestTight(unittest.TestCase):
         m = Model(x, [Tight([x >= 1]),
                       x >= x_min])
         sol = m.solve(verbosity=0)
-        self.assertIs(
-            sol["warnings"]["Unexpectedly Loose Constraints"][0][1], m[0][0])
-        self.assertAlmostEqual(m[0][0].rel_diff, 1, 3)
+        warndata = sol["warnings"]["Unexpectedly Loose Constraints"][0][1]
+        self.assertIs(warndata[-1], m[0][0])
+        self.assertAlmostEqual(warndata[0], 1, 3)
         m.substitutions[x_min] = 0.5
         self.assertAlmostEqual(m.solve(verbosity=0)["cost"], 1)
 
@@ -366,9 +363,9 @@ class TestTight(unittest.TestCase):
         m = Model(x*y, [Tight([x >= y]),
                         x >= 2, y >= 1, sig_constraint])
         sol = m.localsolve(verbosity=0)
-        self.assertIs(
-            sol["warnings"]["Unexpectedly Loose Constraints"][0][1], m[0][0])
-        self.assertAlmostEqual(m[0][0].rel_diff, 1, 3)
+        warndata = sol["warnings"]["Unexpectedly Loose Constraints"][0][1]
+        self.assertIs(warndata[-1], m[0][0])
+        self.assertAlmostEqual(warndata[0], 1, 3)
         m.pop(1)
         self.assertAlmostEqual(m.localsolve(verbosity=0)["cost"], 1, 5)
 
@@ -383,9 +380,9 @@ class TestTight(unittest.TestCase):
                           x >= x_min,
                           y <= y_max])
         sol = m.localsolve(verbosity=0)
-        self.assertIs(
-            sol["warnings"]["Unexpectedly Loose Constraints"][0][1], m[0][0])
-        self.assertGreater(m[0][0].rel_diff, 0.5)
+        warndata = sol["warnings"]["Unexpectedly Loose Constraints"][0][1]
+        self.assertIs(warndata[-1], m[0][0])
+        self.assertGreater(warndata[0], 0.5)
         m.substitutions[x_min] = 0.5
         self.assertAlmostEqual(m.localsolve(verbosity=0)["cost"], 0.5, 5)
 
